@@ -1,8 +1,7 @@
-from downloader_cli.download import Download
-
 name = "downloader_cli"
 
 __all__ = [
             "download",
+            "color",
             "__version__"
         ]
