@@ -4,48 +4,14 @@ import urllib.request
 import sys
 import time
 from os import path, get_terminal_size, name
-import argparse
 import itertools
 from re import match
 
-from downloader_cli.__version__ import __version__
+from downloader_cli.color import ShellColor
 
 # import traceback ## Required to debug at times.
 
 
-def arguments():
-    """Parse the arguments."""
-    parser = argparse.ArgumentParser()
-
-    parser.add_argument('URL', help="URL of the file",
-                        type=str, metavar="SOURCE")
-    parser.add_argument('des', help="target filepath (existing directories \
-                        will be treated as the target location)", default=None, nargs="?",
-                        metavar='TARGET')
-    force = parser.add_mutually_exclusive_group()
-    force.add_argument('-f', '-o', '--force', help="overwrite if the file already exists",
-                       action="store_true")
-    force.add_argument('-c', '--resume', help='resume failed or cancelled \
-                        download (partial sanity check)', action="store_true")
-    parser.add_argument('-e', '--echo', help="print the filepath to stdout after \
-                        downloading (other output will be redirected \
-                        to stderr)", action="store_true")
-    parser.add_argument(
-        '-q', '--quiet', help="suppress filesize and progress info", action="store_true")
-    parser.add_argument(
-        '-b', '--batch', help="Download files in batch. If this flag is passed \
-        the passed source will be considered as a file with download links \
-        seperated by a newline. This flag will be ignored if source is a valid \
-        URL.", default=False, action="store_true"
-    )
-    parser.add_argument('-v', '--version', action='version',
-                        version=__version__,
-                        help='show the program version number and exit')
-
-    args = parser.parse_args()
-    return args
-
-
 class Download:
 
     def __init__(
@@ -59,15 +25,29 @@ class Download:
         batch=False,
         icon_done="▓",
         icon_left="░",
-        icon_border="|"
+        icon_current="",
+        icon_border="|",
+        color_done="",
+        color_left="",
+        color_current=""
     ):
+        # Initialize necessary engines
+        self.__init_color_engine()
+
         self.URL = URL
         self.des = des
         self.passed_dir = None
         self.headers = {}
         self.f_size = 0
-        self.done_icon = icon_done if len(icon_done) < 2 else "▓"
-        self.left_icon = icon_left if len(icon_left) < 2 else "░"
+        self.__done_icon = icon_done if icon_done is not None and len(
+            icon_done) < 2 else "▓"
+        self.__left_icon = icon_left if icon_left is not None and len(
+            icon_left) < 2 else "░"
+        self.__done_color = color_done
+        self.__left_color = color_left
+        self.__current_color = color_current
+        self.__current_icon = icon_current if icon_current is not None and len(
+            icon_current) < 2 else "▓"
         self.border_left, self.border_right = self._extract_border_icon(
             icon_border)
         self._cycle_bar = None
@@ -79,6 +59,25 @@ class Download:
         self.file_exists = False
         self.ostream = sys.stderr if self.echo else sys.stdout
 
+        # Validate the colors
+        self.__validate_passed_color(self.__done_color, 'color_done')
+        self.__validate_passed_color(self.__left_color, 'color_left')
+        self.__validate_passed_color(self.__current_color, 'color_current')
+
+    def __validate_passed_color(self, color, name):
+        if color != "" and not self.color_engine.is_valid_color(color):
+            raise ValueError(f'invalid value passed for `{name}`')
+
+    def __init_color_engine(self):
+        """
+        Initialize the color engine class
+        """
+        self.__color_engine = ShellColor()
+
+    @property
+    def color_engine(self) -> ShellColor:
+        return self.__color_engine
+
     def _extract_border_icon(self, passed_icon):
         """"
         Extract the passed border icon according to
@@ -295,6 +294,44 @@ class Download:
 
         return (next(self._cycle_bar) + 1)
 
+    @property
+    def done_icon(self) -> str:
+        """
+        Return the done icon.
+
+        This will wrap the icon in any colors if they are provided
+        """
+        return self.color_engine.wrap_in_color(self.__done_icon, self.__done_color)
+
+    @property
+    def current_icon(self) -> str:
+        """
+        Return the current icon.
+
+        This will wrap the icon in any colors if they are provided.
+        """
+        return self.color_engine.wrap_in_color(self.__current_icon, self.__current_color)
+
+    @property
+    def left_icon(self) -> str:
+        """
+        Return the left icon.
+
+        This will wrap the icon in any colors if they are provided
+        """
+        return self.color_engine.wrap_in_color(self.__left_icon, self.__left_color)
+
+    def get_done_with_current(self, done_percent: int, remaining_percent: int = None) -> str:
+        if done_percent == 0:
+            return ""
+
+        if remaining_percent == 0:
+            # Download is completed so no need to show the current icon
+            # anymore
+            return self.done_icon * done_percent
+
+        return self.done_icon * (done_percent - len(self.__current_icon)) + self.current_icon
+
     def _get_bar(self, status, length, percent=None):
         """Calculate the progressbar depending on the length of terminal."""
 
@@ -333,13 +370,14 @@ class Download:
                 done = int(percent / (100 / reduce_with_each_iter))
                 status += r"%s%s%s%s" % (
                     self.border_left,
-                    self.done_icon * done,
+                    self.get_done_with_current(
+                        done, reduce_with_each_iter - done),
                     self.left_icon * (reduce_with_each_iter - done),
                     self.border_right)
             else:
                 current_pos = self._get_pos(reduce_with_each_iter)
                 bar = " " * (current_pos - 1) if current_pos > 1 else ""
-                bar += self.done_icon * 1
+                bar += self.current_icon * 1
                 bar += " " * int((reduce_with_each_iter) - current_pos)
                 status += r"%s%s%s" % (self.border_left,
                                        bar, self.border_right)
@@ -445,18 +483,3 @@ class Download:
             self.URL = url
             self._download()
             self.des = self.passed_dir
-
-
-def main():
-    args = arguments()
-    _out = Download(URL=args.URL, des=args.des, overwrite=args.force,
-                    continue_download=args.resume, echo=args.echo,
-                    quiet=args.quiet, batch=args.batch)
-    success = _out.download()
-    if success and args.echo:
-        print(_out.des)
-    sys.stderr.close
-
-
-if __name__ == "__main__":
-    main()
